#!/usr/bin/perl
#
# checker for dnss service by DimmoBorgir
#
#  usage:
#    dnss.checker.pl flag <ip> <uname>
#  or:
#    dnss.checker.pl retrieve <ip> <uname> <flag>
#  or:
#    dnss.checker.pl store <ip> <uname> <flag>
#
#  returns:
#    0 - OK
#    1 - remote malfunction
#    2 - service up, no flag
#    3 - internal error

++$|;

use strict;

my $LOGFILE  = 'dnss.checker.log';
my $EXEC_LOG = 'dnss.checker.exec.log';
my $RETRLOG  = 'dnss.checker.retr.log';
my $FLAG_LEN = 32;

my $CMD_STORE = './dnss.add.pl';				# add './' for nix, remove for win
my $CMD_RETR  = 'nslookup';
my $TIMEOUT   = 5;

$SIG{__WARN__} = sub
{
	my $msg = shift;
	$msg =~ s/[\r\n]//g;
	
	my $strtime = '[ ' . localtime( ). ' ]';
	
	# print "$msg\n";
	
	open LOG, ">> $LOGFILE" or die "fatal error: can't open $LOGFILE\n";
	syswrite LOG, "$strtime  $msg\n";
	close LOG;
};

$SIG{__DIE__} = sub
{
	my ( $pkg, $file, $line ) = caller( );
	warn $_[0].$/;
	print STDERR $_[0];
	print STDERR "died: pkg '$pkg', file '$file', line '$line'\n";
	exit( 3 );
};

my %handlers = (
	'flag'		=> \&flag,
	'retrieve'	=> \&retrieve,
	'store'		=> \&store,
);

warn "\n";

-e $CMD_STORE			or die( "error: '$CMD_STORE' not found\n" );

my $action = shift		or die( "error: no arguments (flag/store/retrieve)\n" ); 
exists $handlers{$action}	or die( "error: invalid action: $action\n" );

$handlers{$action}->( @ARGV );

warn "unknown: exit 3\n";
exit( 3 );

#
# ------------------------------- SUBROUTINES -------------------------------------
#

sub flag		# generate new flag
{
	warn "flag: args = ( @_ )\n";
	@_ == 2 or die "flag: usage: <ip> <uname>\n";
	
	my ( $ip, $uname ) = @_;
	
	my $chars = '1234567890qwertyuiopasdfghjklzxcvbnm';
	my $max_rnd = length( $chars );
	
	my $answ = '';
	$answ .= substr( $chars, int( rand( $max_rnd ) ), 1 ) for 1 .. $FLAG_LEN;
	$answ .= ".$uname.ru";
	
	warn "flag: new flag '$answ'\n";
	print $answ.$/;
	
	warn "flag: exit 0\n";
	exit 0;
}

sub store			# store flag
{
	warn "store: args = ( @_ )\n";
	@_ == 3 or die "store: usage: <store> <ip> <uname> <flag>\n";
	my ( $ip, $uname, $flag ) = @_;
	
	# generate random ip for dns name $flag
	my $ip_for_dns = join '.', map { int( 1+rand(254) ) } ( 1 .. 4 );
	
	my $cmd = "$CMD_STORE $ip $flag $ip_for_dns";
	warn "store: exec: '$cmd'\n";
	
	my ( $ret, $out ) = exec_ex( $cmd );

	$ret == 0 or die "store: external command failed with code $ret\n";
	warn "store: external command completed\n";
	
	# Stored ??
	
	retrieve( @_ );
}

sub retrieve
{
	warn "retrieve: args = ( @_ )\n";
	@_ == 3 or die "retrieve: usage: <retrieve> <ip> <uname> <flag>\n";
	my ( $ip, $uname, $flag ) = @_;
	
	$SIG{ALRM} = sub {
		warn "retrieve: timeout ($TIMEOUT sec)\n";
		warn "retrieve: exit 1\n";
	};
	alarm $TIMEOUT;
	
	my $cmd = "$CMD_RETR $flag $ip";
	
	warn "retrieve: exec: '$cmd'\n";
	
	my ( $ret, $out ) = exec_ex( $cmd );
	$ret == 0 or die "retrieve: external command failed with code $ret\n";
	
	if ( $out =~ /(?:timed out|No response)/ )	# nix and win
	{
		warn "retrieve: no response from server (exit 1)\n";
		retr_action( 'retr', 1, @_ );
		exit 1;
	}
	
	if ( $out =~ /Non-authoritative answer/ )
	{
		warn "retrieve: OK - flag exists (exit 0)\n";
		retr_action( 'retr', 0, @_ );
		exit 0;
	}
	
	if ( $out =~ /can't find/ )
	{
		warn "retrieve: service up, no flag (exit 2)\n";
		retr_action( 'retr', 2, @_ );
		exit 2;
	}
	
	warn "retrieve: unexpected output from external command (exit 3)\n";
	exit 3;
}

sub exec_ex
{
	my $cmd = shift or die "exec_ex: usage: <cmd>\n";
	
	open EXEC_LOG, ">> $EXEC_LOG"	or die "fatal error: can't open '$EXEC_LOG'\n";
	syswrite EXEC_LOG, "-------------------- " . localtime( ) . " ------------------\n";
	syswrite EXEC_LOG, "cmd: $cmd\n";
	
	my $out = `$cmd 2>&1`;
	my $ret = $? >> 8;
	
	syswrite EXEC_LOG, "ret: $ret\n";
	syswrite EXEC_LOG, "out:\n";
	syswrite EXEC_LOG, "$out\n";
	
	close EXEC_LOG;
	
	return ( $ret, $out );
}

sub retr_action
{
	my ( $action, $res, $ip, $uname, $flag ) = @_;
	my $msg = sprintf( "%-5s  %d  %-15s  %-32s  %s\n", $action, $res, $ip, $uname, $flag );
	$msg = '[ '. localtime( ) . ' ] ' . $msg;
	
	open RETRLOG, ">> $RETRLOG" or die "fatal error: can't open '$RETRLOG'\n";
	syswrite RETRLOG, $msg;
	close RETRLOG;
}
